import MuseScore 3.0
import QtQuick 2.0

MuseScore {
    version: "1.0";
    description: "Replaces chord symbols"
    menuPath: "Plugins.ReplaceChordSyms."

    id: replaceChordSyms

    Component.onCompleted : {
        if (mscoreMajorVersion >= 4) {
            replaceChordSyms.title = "Replace Chord Symbols"
        }
     }


    function setCursorToTime(cursor, time) {
        cursor.rewind(0);
        while (cursor.segment) {
            var current_time = cursor.tick;
            if (current_time >= time) {
                return true;
            }
            cursor.next();
        }
        cursor.rewind(0);
        return false;
    }

    function findheadlen(harmony){								
		var str = harmony;							
		var t = str.substr(1,1);							
		if ((t == "b") || (t == "#")){							
			return 2;						
		}							
		else{							
			return 1;						
		}							
	}
    
    
    function findhead(str){
        var headlen = findheadlen(str);					
        return str.substr(0,headlen);					
    }

    function replaceChordSymbols(cursor, segment, toreplace, rwith) {
        console.log ("toreplace = ", toreplace)
        console.log ("rwith = ", rwith)
        

        var headlen = findheadlen(toreplace);	
        var rhead = toreplace.substr(0,headlen)
        var rtail = toreplace.substr(headlen)
        headlen = findheadlen(rwith);	
        var whead = rwith.substr(0,headlen)
        var wtail = rwith.substr(headlen)
        
                     
        //console.log ("rhead = ", rhead)
        //console.log ("rtail = ", rtail)
        //console.log ("whead = ", whead)
        //console.log ("wtail = ", wtail)
        
        while (segment) {
            var aCount = 0;
            var annotation = segment.annotations[aCount];

            while (annotation) {
                if (annotation.type == Element.HARMONY) {
                    var head = "";
                    var tail = ""
                    var slashtail = "";
                    var harmony = annotation;
                    var str = harmony.text;
                    if (str != ""){
                        //console.log ("str = ", str  + " " + segment.tick) 
                        // find slashtail
                        var slashtail = "";
                        var n = str.search("/");					
                        if (n > -1) {
                            // only for /note
                            //slashtail = str.substr(n);
                            //head = str.substr(0,n);
                            //console.log ("slashtail = ", slashtail)  
                            //console.log ("head = ", head)          
                        }
                        // relevant cases:
                        // rhead != whead -> global enharmonic substitution
                        if (rhead != whead){
                            var str = harmony.text;
                            var n = str.search("/");					
                            if (n > -1) str = str.substr(0,n);

                            if(str == toreplace) {
                                if (slashtail.length > 0)   harmony.text = rwith + slashtail;
                                else harmony.text = rwith;
                                harmony.text = rwith;
                                slashtail = "";
                                console.log ("Replacing " + toreplace + slashtail + " with " + rwith + slashtail + " at tick " + segment.tick)
                            } 
                        }
                        // rhead = whead & rtail != wtail -> tail substitution
                        else{
                            // no slash was found in the input
                            //if (head == ""){
                                headlen = findheadlen(str);					
                                head = str.substr(0,headlen);					
                            //}        
                            //console.log ('head = ', head);	
                            tail = str.substr(headlen,(str.length - headlen))					
                            tail = str.substr(headlen);
                            //console.log ('139 tail = ', tail);	
                            var n = tail.search("/");					
                            slashtail = "";
                            if (n > -1) {
                                slashtail = tail.substr(n);
                                //tail = tail.substr(0,n);
                            }
                            //console.log ('146 tail = ', tail);
                            if(tail == rtail) {
                                slashtail = "";
                                harmony.text = head + wtail + slashtail;
                                console.log ("Replacing " + str + " with " + head + wtail + slashtail + " at tick " + segment.tick)
                            } // if(tail == rtail)
                        } // end else	
                    }// end if str
                }// end annotation.type == Element.HARMONY
                annotation = segment.annotations[++aCount];
            } // end while annotation   
            segment = segment.next;    
        } // end while segment    
    } // end function

function smartQuit() {
		
		if (mscoreMajorVersion < 4) {Qt.quit()}
		else {quit()}
	}//smartQuit
    

    onRun: {
        if (typeof curScore === 'undefined') {
            smartQuit();
        }
        //["from", "to"], // comment
        var replist = [
["A#","Bb"], // unique
["A#dim","Bbdim"], // unique
["A#m7","Bbm7"], // unique
["A#Maj7#5","BbMaj#5"], // unique
["A#mMaj7","BbmMaj7"], // unique
["Ab7(#5)","Ab7b13"], // unique
["Ab9","Ab7"], // alle n(ote)9 with n7
["Abadd9","Ab"], // alle nadd9 with n
["Abm","Abm7"], // alle n with nm7 (except ##)
["Abm(Maj7)","AbmMaj7"],
["Abmadd9","Abm7"], // alle nmadd9 with nm
["AbMaj7#5","AbMaj#5"], // alle Maj7#5 with Maj#5
["AbMaj7(#5)","AbMaj#5"],
["AbMaj9","Ab"], // alle Maj9 with ""
["AbmMaj7(b5)","Bdim"],
["AMaj7","A"], // alle Maj9 with ""
["AMaj7#11","AMaj#11"], // alle Maj7#11 with Maj#11
["AmMaj7(#5)","F"], // unique
["B#","C"],// unique
["B#+","C+"],// unique
["B#dim","Cdim"],// unique
["B#m7","Cm7"],// unique
["B#Maj7#11","CMaj#11"],
["B#mMaj7","CmMaj7"],// unique
["B#o7","D7b9"],
["B11(b9/b13)","B7sus"],
["Bb7#5","Bb7b13"],// alle 7#5 with 7b13
["Bb7(#9)","Bb7#9"],
["BbbmMaj7(b5)","Bb7sus"],
["Bbm11","Bbm7"], // alle m11 c. m7
["Bbm7(11)","Bbm7"],
["Bbm9","Bbm7"], // alle m9 c. m7
["BbMaj7#11","BbMaj#11"],
["BbmMaj7(#5)","BbmMaj7"], // alle mMaj7(#5) c. mMaj7
["BbmMaj7(b5)","BbmMaj7"], // alle mMaj7(b5) c. mMaj7
["C#","Db"], //unique
["C##7","D7"], //unique
["C##dim","Ddim"],
["C#+","Db+"],//unique
["C#Maj7#11","C#Maj#11"], // al gehad?
["B#Maj#11","CMaj#11"],  // unique
["C#Maj#11","DbMaj#11"], //unique
["C#o7","Gdim"],
["C+","CMaj#5"],
["C7(#5)","C7b13"],
["C7(b5)","C7#11"],
["Cb","B"],//unique
["Cbdim","Bdim"],// al gehad?
["Cbm","Bm7"],// al gehad?
["Cbm7","Bm7"],// al gehad?
["CbMaj7","B"],// al gehad?
["CbmMaj7","BmMaj7"],
["Co7","Ebdim"],
["D#","Eb"],//unique
["D#+","Eb+"],//unique
["D#7b5","Eb7#11"],//unique
["D#dim","Ebdim"],//unique
["D#m7","Ebm7"],//unique
["D#mMaj7","EbmMaj7"],//unique
["D11(b9/b13)","D7susb9"],
["Db7b5","Db7#11"],// alle 7b5 c. 7#11
["Dbdim","C#dim"],
["DMaj7#11","DMaj#11"],
["DmMaj7(b5)","Adim"],
["Do7","Bdim"],
["E#","F"], //unique
["E#m7","Fm7"],//unique
["E7#5","E7b13"],
["E7b5","E7#11"],
["Eb11(b9/b13)","Eb7sus"],
["Eb7(b9)","Eb7b9"],
["EbMaj7#5","EbMaj#5"],
["EbMaj7(#11)","Eb7#11"],
["EMaj7#5","EMaj#5"],
["F#7#5","F#7b13"],
["F##m","Gm7"], //unique
["F#Maj7#5","F#Maj#5"],
["F#Maj#5","GbMaj#5"],//unique
["F07","Fm7b5"], // all 07 c. m7b5
["F7(13)","F7"], // all 7(13) c. 7
["F9#11","F7#11"],// all 9#11 c. 7#11
["Fb","E"],//unique
["Fb7","E7"],//unique
["FMaj7#11","FMaj#11"],
["G#","Ab"],
["G#7(#5)","Ab7b13"],
["G#dim","Abdim"],
["G7#5","G7b13"],
["G7(#5)","G7b13"], 
["Gm","Gm"],
["GMaj7#11","GMaj#11"],
["GMaj7#5","GMaj#5"],
["A#7#9","Bb7#9"], //unique
["Abm9Maj7","AbmMaj7"],// alle m9M
["AbMaj9#11","AbMaj#11"],// alle Maj9#11
["B#7","C7"], //unique
["Bbm9Maj7","BbmMaj7"],
["BMaj9#11","BMaj#11"],
["C##Maj7","D"],//unique
["C#7#9","Db7#9"],//unique
["C#7b13","Db7b13"],//unique
["C#7b9","Db7b9"],//unique
["C#m9Maj7","DbmMaj7"],
["C#Maj#5","DbMaj#5"],//unique
["C#Maj9#11","DbMaj#11"],
["C9(13)","C7"],// all 9(13)
["Cb7#9","B7#9"],//unique
["D#Maj#11","EbMaj#11"],//unique
["Dbm9Maj7","DbmMaj7"],
["Dm9Maj7","DmMaj7"],
["DMaj9#11","DMaj#11"],
["E#7","F7"],//unique
["E#7#11","F7#11"],//unique
["E#7#9","F7#9"],//unique
["E#7b13","F7b13"],//unique
["E#dim","Fdim"],//unique
["E#m7b5","Fm7b5"],//unique
["E#Maj#11","FMaj#11"],//unique
["E#mMaj7","FmMaj7"],//unique
["Em9Maj7","EmMaj7"],
["F#9(13)","F#7"],
["F#Maj9#11","F#Maj#11"],
["Fm9Maj7","FmMaj7"],
["FMaj9#11","FMaj#11"],
["G#","Ab"],
["G#m9Maj7","AbmMaj7"],
["G#Maj#5","AbMaj#5"], //unique
["Gm9Maj7","GmMaj7"],
["GMaj9#11","GMaj#11"],
["A11(b9/b13)","A7susb9"],
["A7b9b13","A7b13"],
["C##mMaj7(#5)","D7susb9"],
["C#7b9b13","Db7b13"],
["Ddim7","Ddim"],
["F#Maj#11","GbMaj#11"],
];

        var cursor = curScore.newCursor();
        //var j = 275 - 148;
        for (var i in replist){
        //for (var i = j; i < j + 5; i++){
            cursor.rewind(0); // beginning of score									
            var segment = curScore.firstSegment();
            curScore.startCmd();
            //console.log ("processing = ", replist[i][0])
            //console.log ("replist[i][1] = ", replist[i][1])
            replaceChordSymbols(cursor, segment, replist[i][0],replist[i][1])    
            curScore.endCmd();
        }
        smartQuit()    
        } // end onRun					
}
